@@ -20,6 +20,18 @@ our @EXPORT_OK = qw(
 );
 use constant GSL_IS_WINDOWS =>  ($^O =~ /MSWin32/i)  ?  1 : 0 ;
 
+=head1 NAME
+
+Math::GSL::Test - Assertions and such
+
+=head1 SYNOPSIS
+
+
+    use Math::GSL::Test qw/:all/;
+    ok_similar($x,$y, $msg, $eps);
+
+=cut
+
 our %EXPORT_TAGS = ( 
                      all => \@EXPORT_OK,
                    );
@@ -31,8 +43,33 @@ sub _dump_result($)
     printf "result->val: %.18g\n", $r->{val};
 }
 
+=head2 is_windows()
+
+Returns true if current system is Windows-like.
+
+=cut
+
 sub is_windows() { GSL_IS_WINDOWS }
 
+=head2 is_similar($x,$y;$eps,$similarity_function)
+    
+    is_similar($x,$y);
+    is_similar($x, $y, 1e-7);
+    is_similar($x,$y, 1e-3, sub { ... } );
+
+Return true if $x and $y are within $eps of each other, i.e. 
+
+    abs($x-$y) <= $eps 
+
+If passed a code reference $similarity_function, it will pass $x and $y as parameters to it and 
+will check to see if 
+
+    $similarity_function->($x,$y_) <= $eps
+
+The default value of $eps is 1e-8. Don't try sending anything to the moon with this value...
+
+=cut
+
 sub is_similar {
     my ($x,$y, $eps, $similarity_function) = @_;
     $eps ||= 1e-8;
@@ -57,7 +94,7 @@ sub is_similar {
                $similarity_function->($x,$y) <= $eps ? return 1 : return 0;
         } elsif( defined $x && defined $y) { 
             my $delta = (gsl_isnan($x) or gsl_isnan($y)) ? gsl_nan() : abs($x-$y);
-            $delta > $eps ? return 0 : return 1;
+            $delta > $eps ? warn qq{\t\t\$x=$x\n\t\t\$y=$y\n\t\tdelta=$delta\n} && return 0 : return 1;
         } else {
             return 0;
         }
@@ -103,6 +140,28 @@ sub verify_results
         }
     }
 }
+
+=head2 verify( $results, $class) 
+
+Takes a hash reference of key/value pairs where the keys are bits of code, which when evaluated should
+be within some tolerance of the value. For example:
+
+    my $results = { 
+                    'gsl_cdf_ugaussian_P(2.0)'        => [ 0.977250, 1e-5 ],
+                    'gsl_cdf_ugaussian_Q(2.0)'        => [ 0.022750, 1e-7 ],
+                    'gsl_cdf_ugaussian_Pinv(0.977250)'=> [ 2.000000 ],
+                  };
+    verify($results, 'Math::GSL::CDF');
+
+When no tolerance is given, a value of 1e-8 = 0.00000001 is used. One
+may use $GSL_NAN and $GSL_INF in comparisons and this routine will
+use the gsl_isnan() and gsl_isinf() routines to compare them.
+
+
+Note: Needing to pass in the class name is honky. This may change.
+
+=cut
+
 sub verify
 {
     my ($results,$class) = @_;
@@ -112,40 +171,78 @@ sub verify
         my $x = eval qq{${class}::$code};
         ok(0, $@) if $@;
 
-        my ($expected,$eps);
-        if (ref $result){
-            ($expected,$eps)=@$result;
-        } else {
-            ($expected,$eps)=($result,1e-8);
-        }
-        my $res = abs($x - $expected);
+        croak(__PACKAGE__." : $result is not an array reference!") unless ref $result;
+        my($expected,$eps)=@$result;
+        $eps ||= 1e-8;
 
         if (gsl_isnan($x)) {
                ok( gsl_isnan($expected), "'$expected'?='$x'" );
         } elsif(gsl_isinf($x)) {
                ok( gsl_isinf($expected), "'$expected'?='$x'" );
         } else {
+            my $res = abs($x - $expected);
             ok( $res <= $eps, "$code ?= $x,\nres= +-$res, eps=$eps" );    
         }
     }
 }
+
+=head2 ok_status( $got_status; $expected_status )
+
+    ok_status( $status );                  # defaults to checking for $GSL_SUCCESS
+
+    ok_status( $status, $GSL_ECONTINUE );
+
+Pass a test if the GSL status codes match, with a default expected status of $GSL_SUCCESS. This
+function also stringifies the status codes into meaningful messages when it fails.
+
+=cut
+
 sub ok_status {
     my ($got, $expected) = @_;
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     $expected ||= $GSL_SUCCESS;
-    ok( $got == $expected, gsl_strerror(int($got)) );
+    ok( defined $got && $got == $expected, gsl_strerror(int($got)) );
 }
+
+=head2 ok_similar( $x, $y, $msg, $eps)
+
+    ok_similar( $x, $y);
+    ok_similar( $x, $y, 'reason');
+    ok_similar( $x, $y, 'reason', 1e-4);
+
+Pass a test if is_similar($x,$y,$msg,$eps) is true, otherwise fail.
+
+=cut
+
 sub ok_similar {
     my ($x,$y, $msg, $eps) = @_;
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     ok(is_similar($x,$y,$eps), $msg);
 }
 
+=head2 is_similar_relative( $x, $y, $msg, $eps )
+
+    is_similar_relative($x, $y, $eps );
+
+Returns true if $x has a relative error with repect to $y less than $eps. The
+current default for $eps is the same as is_similar(), i.e. 1e-8. This doesn't
+seem very useful. What should the default be?
+
+=cut
+
 sub is_similar_relative {
     my ($x,$y, $eps) = @_;
     return is_similar($x,$y,$eps, sub { abs( ($_[0] - $_[1])/abs($_[1]) ) } );
 }
 
+=head2 ok_similar_relative( $x, $y, $msg, $eps )
+
+    ok_similar_relative($x, $y, $msg, $eps );
+
+Pass a test if $x has a relative error with repect to $y less than $eps.
+
+=cut
+
 sub ok_similar_relative {
     my ($x,$y, $msg, $eps,) = @_;
     local $Test::Builder::Level = $Test::Builder::Level + 1;
